\\ Copyright 2015, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
read("terdragon-defines.gp")

check_count = 0;
check_exact(got,want,name="") =
{
  check_count++;
  if(got!= want,
     print("wrong "name);
     print("got  "got);
     print("want "want);
     quit(1),
     print("ok "check_count" "name));
}

\\-----------------------------------------------------------------------------
\\ find maximums

factor_vector(n) = concat(apply((r)->vector(r[2],i,r[1]), Vec(Col(factor(n)))));

P_funcs = [P1,P2,P3,P4,P5,P6,P7, \
           P1dash,P2dash,P3dash,P4dash,P5dash,P6dash,P7dash];
P(k,n) = P_funcs[n](k);
list=List([]);
{
  for(k=6,20,
     my(max_width = 0,
        max_a=-1,
        max_b=-1,
        prev_max_a=-1,
        prev_max_b=-1);
     for(a=1,10,
        for(b=a+1,10,
              my(width = norm(P(k,a) - P(k,b)));
              if(width>max_width,
                 prev_max_a = max_a;
                 prev_max_b = max_b;
                 max_width = width;
                 max_a = a;
                 max_b = b)));
     print("k="k"  "max_a" "max_b" width "max_width" ="factor_vector(max_width),
          "  second max ",prev_max_a," ",prev_max_b);
     listput(list,max_width);
  );
}
Vec(list)

\\-----------------------------------------------------------------------------

P3_sub_P3dash(k) =
{
  -1/24*(b^(k+2)+p(k+2))
  - (b^k - (-1/24)*(b^(k+2)+p(k+2)));
}
check_exact(vector(64,k,k+=5; P3_sub_P3dash(k)), \
            vector(64,k,k+=5; P3(k)-P3dash(k)));

P3_sub_P3dash(k) =
{
  - b^k - 1/12*(b^(k+2)+p(k+2));
}
check_exact(vector(64,k,k+=5; P3_sub_P3dash(k)), \
            vector(64,k,k+=5; P3(k)-P3dash(k)));

P3_sub_P3dash(k) =
{
  -(9/8 + 1/8*sqrt3i)* b^k - 1/12*p(k+2);
}
check_exact(vector(64,k,k+=5; P3_sub_P3dash(k)), \
            vector(64,k,k+=5; P3(k)-P3dash(k)));

norm_P3_sub_P3dash(k) =
{
  norm( (9/8 + 1/8*sqrt3i)* b^k + 1/12*p(k+2) );
}
check_exact(vector(64,k,k+=5; norm(P3_sub_P3dash(k))), \
            vector(64,k,k+=5; norm(P3(k)-P3dash(k))));

\\ P3dash(k) = b^k - P3(k);
\\ vector(200,k,k+=5; norm(P3(k)-P3dash(k)))

gNorm(x) = {
     + -1 - 3*x - 9*x^2 - 31*x^3 - 103*x^4 - 309*x^5

     + (1/2)/(1 - x)
     + (-3/16)/(1 + x)
     + (-1/4 - 1/2*x)/(1 + x^2)

     + (-3/4 - 3/2*x)/(1 - 3*x^2)
     + (3/8 + 3/8*x)/(1 + 3*x^2)

     + (21/16)/(1 - 3*x);
}
check_exact(gf_terms(gNorm(x),200), \
            vector(200,k,k--; if(k<6,0, norm(P3(k)-P3dash(k)))));

\\
{
  check_exact(gf_terms(     + (1/2)/(1 - x)
                       + (-3/16)/(1 + x)
                       + (-1/4 - 1/2*x)/(1 + x^2)
                       ,20), \
              vector(20,k,k--; vector_modulo([
                                              1/16, 3/16, 9/16, 19/16
                                              ],k)));
}

\\ -3/8, -9/8, -9/8, -15/8
{
  my(v = gf_terms(+ (-3/4 - 3/2*x)/(1 - 3*x^2)
                  + (3/8 + 3/8*x)/(1 + 3*x^2),
                  20));
  for(i=1,#v,
     my(k=i-1);
     \\ print(v[i]/3^floor(k/2));
  );
}

norm_P3_sub_P3dash(k) =
{
  21/16 * 3^k
  - 1/8*vector_modulo([3, 9, 9, 15],k) * 3^floor(k/2)
  + 1/16*vector_modulo([1, 3, 9, 19],k);
}
check_exact(vector(200,k,k+=5; norm(P3_sub_P3dash(k))), \
            vector(200,k,k+=5; norm(P3(k)-P3dash(k))));

core_of_frac(f) = core(f*denominator(f)^2);
smallest_prime_factor(n) = factor(n)[1,1];

print("squares");
for(k=3,40,print(smallest_prime_factor(core_of_frac(norm_P3_sub_P3dash(k)))));
\\ vector(100,k,k--; norm_P3_sub_P3dash(k))
\\ vector(100,k,k--; core_of_frac(norm_P3_sub_P3dash(k)))
\\ vector(100,k,k--; norm_P3_sub_P3dash(k)/core_of_frac(norm_P3_sub_P3dash(k)))

\\-----------------------------------------------------------------------------
\\ jacobi symbol for non-squareness
HD = norm_P3_sub_P3dash;

print("\njacobi");
print("k mod 256 ",vector(20,k,my(k=k+3); HD(k)%256));

try(q) =
{
  my(v=vector(20,k,my(k=k+3); kronecker(HD(k),q)));
  my(undecided=sum(i=1,#v, v[i]!=-1));
  print("q="q" ",if(undecided,""," [good] "),v);
}

for(q=2,150, if(isprime(q), try(q)))
try(256)