\\ Copyright 2014, 2017 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.



nearly_equal(x,y) = abs(x-y) < 1e-6
test_equal(x,y,name="") = if(!nearly_equal(x,y),print(name," bad ",x," ",y);quit(1))
deg(x) = x/180*Pi

nearly(x) = if(type(x)=="t_VEC",apply(nearly,x), \
               my(v=x*16); if(nearly_equal(v,round(v)), round(v)/16, x))
asdeg(x) = nearly(x/Pi*180)

\\ 0, 30
for(i=0,256,my(alpha=i/256*Pi); test_equal(sin(alpha+deg( 60))^2, -sin(alpha)^2 + sin(alpha+deg(30))^2 + 1/2, i))
for(i=0,256,my(alpha=i/256*Pi); test_equal(sin(alpha+deg( 90))^2, -sin(alpha)^2                        + 1,   i))
for(i=0,256,my(alpha=i/256*Pi); test_equal(sin(alpha+deg(120))^2,               - sin(alpha+deg(30))^2 + 1,   i))
for(i=0,256,my(alpha=i/256*Pi); test_equal(sin(alpha+deg(150))^2,  sin(alpha)^2 - sin(alpha+deg(30))^2 + 1/2, i))

\\ 0, 60
for(i=0,256,my(alpha=i/256*Pi); test_equal(sin(alpha+deg( 30))^2,  sin(alpha)^2 + sin(alpha+deg(60))^2 - 1/2  ))
for(i=0,256,my(alpha=i/256*Pi); test_equal(sin(alpha+deg( 90))^2, -sin(alpha)^2                        + 1,   i))
for(i=0,256,my(alpha=i/256*Pi); test_equal(sin(alpha+deg(120))^2, -sin(alpha)^2 - sin(alpha+deg(60))^2 + 3/2, i))
for(i=0,256,my(alpha=i/256*Pi); test_equal(sin(alpha+deg(150))^2,               - sin(alpha+deg(60))^2 + 1,   i))

w6 =  1/2 + sqrt(3)/2*I
w3 = -1/2 + sqrt(3)/2*I
b = w3 + 2

terdragon(n) = my(v=digits(n,3),ret=0,dir=1); \
               for(i=1,#v, ret*=b; if(v[i]==1, ret+=dir;dir*=w3, v[i]==2, ret+=dir*w6)); ret
test_equal(terdragon(0), 0)
test_equal(terdragon(1), 1)
test_equal(terdragon(2), w6)
test_equal(terdragon(3), w6+1)

terdragon_midpoint(n) = (terdragon(n) + terdragon(n+1))/2
terdragon_centroid(k) = b^k/2

test_equal(terdragon_midpoint(0), 1/2)

\\ p = vector(27,n,my(n=n-1); terdragon(n))
\\ plothraw(apply(real,p),apply(imag,p), 1); \\ flag=1 join points

DEBUG=0
inertia(k,alpha) =
{
  my(r=exp(I*(-alpha - k*deg(30))));
  if(DEBUG,print("k=",k," alpha=",asdeg(alpha)," r=",r));
  my(c=terdragon_centroid(k));
  sum(n=0,3^k-1,
      if(DEBUG,print("point ",n," at ",r*(terdragon_midpoint(n) - c)));
      imag( r*(terdragon_midpoint(n) - c) )^2
  );
}

\\        *
\\       /                   *__
\\      /                    |  --__
\\     *--+--*        *__    |      *
\\          /            --__|
\\         /                 *
\\        *
\\
\\     *-------*
\\      \
\\       +
\\        \
\\ *---+---*
\\
Il = 2 * ( 1/2 * sin(deg(30))  )^2
\\ 1/2 * sin(deg(30))
test_equal(inertia(1,0), Il)


inertia_by_rec(k,alpha) =
{
  if(k==0,return(0));

  return(inertia(k-1, alpha - deg(90))
     + 2*inertia(k-1, alpha + deg(30))
     + 1/2 * 9^(k-1) * sin(alpha - deg(30))^2);

  inertia(k-1, alpha - deg(90))
  + 2*inertia(k-1, alpha + deg(30))
  + 1/2 * 9^(k-1) * sin(alpha - deg(30))^2
  \\ + 2 * 3^(k-1) * ( 1/2*sqrt(3)^(k-1) * sin(deg(30)-alpha) )^2
  ;
}


\\ my(k=1,alpha=0); 2 * 3^(k-1) * ( 1/2*sqrt(3)^(k-1) * sin(deg(30)-alpha) )^2
\\ inertia_by_rec(1,0)
\\ inertia(1,0)

\\
\\       *---+---*
\\        \
\\         +                         .       .
\\          \                       / \     / \
\\   *---+---*                     +   +   +   +
\\    \     / \                   /     \ /     \
\\     +   +   +         .       . --+-- .       *
\\      \ /     \         \     / \     /
\\       *---+---*         +   +   +   +
\\        \                 \ /     \ /
\\         +                 .       .
\\          \
\\   *---+---*
\\
Il = 8*( 1/2*sin(deg(60)) )^2  * 8



\\ inertia(k,alpha) = A(k) * sin(alpha)^2
\\                  + B(k) * sin(alpha+deg(30))^2
\\                  + C(k)

\\ inertia(k,alpha) =   inertia(k-1, alpha - deg(90))
\\                  + 2*inertia(k-1, alpha + deg(30))
\\                  + 1/2 * 9^(k-1) * sin(alpha - deg(30))^2);
\\ =
\\       A * sin(alpha         - deg(90))^2
\\   +   B * sin(alpha+deg(30) - deg(90))^2
\\   +   C
\\   + 2*A * sin(alpha         + deg(30))^2
\\   + 2*B * sin(alpha+deg(30) + deg(30))^2
\\   + 2*C
\\   + 1/2 * 9^(k-1) * sin(alpha - deg(30))^2
\\
\\ =
\\       A * sin(alpha + deg(90) )^2
\\   +   B * sin(alpha + deg(120))^2
\\   +   C 
\\   + 2*A * sin(alpha + deg(30))^2
\\   + 2*B * sin(alpha + deg(60))^2
\\   + 2*C
\\   + 1/2 * 9^(k-1) * sin(alpha + deg(150))^2
\\
\\ 0, 30
\\ sin(alpha+deg(60))^2  = - sin(alpha)^2 + sin(alpha+deg(30))^2 + 1/2
\\ sin(alpha+deg(90))^2  = - sin(alpha)^2                        + 1
\\ sin(alpha+deg(120))^2 =                - sin(alpha+deg(30))^2 + 1
\\ sin(alpha+deg(150))^2 =   sin(alpha)^2 - sin(alpha+deg(30))^2 + 1/2
\\
\\       A       * (- sin(alpha)^2                        + 1 )       +90
\\   +   B       * (               - sin(alpha+deg(30))^2 + 1 )       +120
\\   +   C
\\   + 2*A       *                   sin(alpha+deg(30))^2             +30
\\   + 2*B       * (- sin(alpha)^2 + sin(alpha+deg(30))^2 + 1/2 )     +60
\\   + 2*C
\\   + 1/2*9^(k-1)*( sin(alpha)^2 - sin(alpha+deg(30))^2 + 1/2 )     +150
\\
\\   sin(alpha)^2           * (  -A(k-1) - 2*B(k-1) + 1/2*9^(k-1)
\\   sin(alpha + deg(30))^2 * ( 2*A(k-1) +   B(k-1) - 1/2*9^(k-1)
\\   + 3*C(k-1) + A(k-1) + 2*B(k-1) + 1/4*9^(k-1)
\\
A(k) = if(k==0,0,            -A(k-1) - 2*B(k-1) + 1/2*9^(k-1))
B(k) = if(k==0,0,           2*A(k-1) +   B(k-1) - 1/2*9^(k-1))
C(k) = if(k==0,0, 3*C(k-1) +  A(k-1) + 2*B(k-1) + 1/4*9^(k-1))
E(k) = 0
inertia_by_ABC(k,alpha) =
{
    A(k)*sin(alpha)^2
  + B(k) * sin(alpha + deg(30))^2
  + E(k) * sin(alpha + deg(60))^2
  + C(k);
}
       
\\ A(k) =  -A(k-1) - 2*B(k-1) + 1/2*9^(k-1)
\\ B(k) = 2*A(k-1) +   B(k-1) - 1/2*9^(k-1)

\\ B(k-1) = - 1/2*A(k) - 1/2*A(k-1) + 1/4*9^(k-1)

\\ - 1/2*A(k+1) - 1/2*A(k) + 1/4*9^k = 2*A(k-1) - 1/2*A(k) - 1/2*A(k-1) + 1/4*9^(k-1) - 1/2*9^(k-1)
\\ - 1/2*A(k+1) = 3/2*A(k-1) - 5/2*9^(k-1)
\\ A(k+1) = - 3*A(k-1) + 5*9^(k-1)
\\ A(k) = - 3*A(k-2) + 5*9^(k-2)
\\

\\ A(k) =  -A(k-1) - 2*B(k-1) + 1/2*9^(k-1)
\\
\\ A(k-1) = 1/2*B(k) - 1/2*B(k-1) + 1/4*9^(k-1)
\\ 1/2*B(k+1) - 1/2*B(k) + 9/4*9^(k-1) = -(1/2*B(k) - 1/2*B(k-1) + 1/4*9^(k-1)) - 2*B(k-1) + 1/2*9^(k-1)
\\ 1/2*B(k+1) = -3/2*B(k-1) - 2*9^(k-1)
\\ B(k+1) = -3*B(k-1) - 4*9^(k-1)
\\ B(k) = -3*B(k-2) - 4*9^(k-2)

A(k) = if(k==0,0, k==1, 1/2, -3*A(k-2) + 5*9^(k-2))
B(k) = if(k==0,0, k==1,-1/2, -3*B(k-2) - 4*9^(k-2))

vector(8,k,A(k-1))
\\ vector(8,k,B(k-1))

Ae(h) = if(h==0,0,   -3*Ae(h-1) + 5*81^(h-1));
Ao(h) = if(h==0,1/2, -3*Ao(h-1) + 5*9*81^(h-1));

\\ A(4) = -3*A(2) + 5*9^2
\\      = -3*(-3*A(0) + 5*9^0) + 5*9^2
\\      = -3*( 5*9^0) + 5*9^2 = 390
\\ Ae(h) = 5*81^(h-1) + (-3)*5*81^(h-2) + ... + (-3)^(h-1)*5*81^0 + (-3)^h*Ae(0)
\\       = 5*( 81^h - (-3)^h ) / (81 - (-3)) 
Ae(h) = 5/84*( 81^h - (-3)^h )

\\ Ao(1) = -3*1/2 + 45*81^0
\\ Ao(h) = 45*81^(h-1) + (-3)*45*81^(h-2) + ... + (-3)^(h-1)*45*81^0 + (-3)^h*Ao(0)
\\       = 45*( 81^h - (-3)^h ) / (81 - (-3))  + (-3)^h*1/2
\\ Ao(h) = 45*( 81^h - (-3)^h ) / (81 - (-3))  + (-3)^h*1/2
Ao(h) = 3/84*(15*81^h - (-3)^h)
A(k) = if(k%2==0, Ae(k/2), Ao((k-1)/2))

A(k) = if(k%2==0, 5/84*( 81^(k/2) - (-3)^(k/2) ), 3/84*(15*81^((k-1)/2) - (-3)^((k-1)/2)) )
A(k) = 5/84*9^k - 1/84*(4+(-1)^k)*(-3)^floor(k/2);
A(k) = 5/84*9^k - if(k%2==0,5/84,3/84)*(-3)^floor(k/2);

Be(h) = -4*( 81^h - (-3)^h ) / (81 - (-3)) 
Bo(h) = -4*9*( 81^h - (-3)^h ) / (81 - (-3))  + (-3)^h*-1/2

Be(h) = -4/84*( 81^h - (-3)^h );
Bo(h) = -36/84*( 81^h - (-3)^h )  + (-3)^h*-1/2;
Bo(h) = -36/84*( 81^h - (-3)^h  + (-3)^h*-1/2*-84/36 );
Bo(h) = -36/84*( 81^h - (-3)^h  +  7/6*(-3)^h );
Bo(h) = -36/84*( 81^h + 1/6*(-3)^h );
B(k) = if(k%2==0, -4/84*( 81^(k/2) - (-3)^(k/2) ), -36/84*( 81^((k-1)/2) + 1/6*(-3)^((k-1)/2) ) );
B(k) = if(k%2==0, -4/84*81^(k/2) + 4/84*(-3)^(k/2), -36/84*81^((k-1)/2) + -36/84*1/6*(-3)^((k-1)/2) );
B(k) = -4/84*9^k + if(k%2==0,4/84,-6/84)*(-3)^floor(k/2);

vector(8,k,A(k-1))
\\ vector(8,k,B(k-1))
\\ vector(8,h,B(2*h+1))
vector(8,k,A(k-1)-B(k-1))

vector(12,k,C(k-1))
\\ C(k) = A(k-1) + 2*B(k-1) + 1/4*9^(k-1)  +  3*C(k-1)
\\ C(k) = A(k-1) + 2*B(k-1) + 1/4*9^(k-1)  +  3*(A(k-2) + 2*B(k-2) + 1/4*9^(k-2) + 3*())
\\ const 1/4*9^(k-1) + 3*1/4*9^(k-2) + ... + 3^(k-1)*1/4*9^0
\\     = 1/4*(9^k - 3^k) / (9 - 3)
\\     = 1/24*(9^k - 3^k)
\\ Ao(h)+2*Bo(h) = 
\\               = 
\\ ABeo(h) = -3/84*(10*81^h + 4*(-3)^h)
\\ ABeo9s(h) = -3/84*(10*81^h + 4*(-3)^h) + 9*() + 9^2*() + ... + 9^h*-3/84*(10+4)
\\           = -3/84*10*(81^(h+1) - 9^(h+1))/(81-9)    + -3/84*4*((-3)^(h+1) - 9^(h+1))/((-3)-9) 
\\           = -5/1008*81^(h+1) - -5/1008*9^(h+1)  + 1/84*(-3)^(h+1) - 1/84*9^(h+1)
\\           = -5/1008*81^(h+1) - 1/144*9^(h+1)  + 1/84*(-3)^(h+1)
\\           = -5/1008*81^(h+1) - 1/144*9^(h+1) + 1/84*(-3)^(h+1)
\\
\\ Apart =       5/84*9^k     - if(k%2==0,5/84,3/84)*(-3)^floor(k/2);
\\         + 3*( 5/84*9^(k-1) - if((k-1)%2==0,5/84,3/84)*(-3)^floor((k-1)/2)
\\         + ...
\\         + 3^(k-1)*( 5/84*9^0 - if(0%2==0,5/84,3/84)*(-3)^0

ABe(h) = Ae(h) + 2*Be(h);
ABe(h) = 5/84*( 81^h - (-3)^h ) + 2*-4/84*( 81^h - (-3)^h );
ABe(h) = -3/84*( 81^h - (-3)^h );

ABo(h) = Ao(h) + 2*Bo(h);
ABo(h) = 3/84*(15*81^h - (-3)^h) + 2*-36/84*( 81^h + 1/6*(-3)^h );
ABo(h) = -3/84*(9*81^h + 5*(-3)^h);

ABoe(h) = ABo(h) + 3*ABe(h);
ABoe(h) = -3/84*9*81^h + -3/84*5*(-3)^h - 9/84*81^h + 9/84*(-3)^h;
ABoe(h) = -3/7*81^h - 1/14*(-3)^h;

C(k) = sum(i=0,k-1, 3^(k-1-i)*(A(i)+2*B(i)+1/4*9^(i)));
C(k) = sum(i=0,k-1, 3^(k-1-i)*(A(i)+2*B(i))) + sum(i=0,k-1, 3^(k-1-i)*1/4*9^(i));
C(k) = sum(i=0,k-1, 3^(k-1-i)*(A(i)+2*B(i))) + 1/24*(9^k - 3^k);
C(k) =
{
  my(h=floor(k/2));
     1/24*(9^k - 3^k) + if(k%2==0,
                           sum(i=0,h-1, 9^(h-1-i)*(Ao(i)+2*Bo(i) + 3*Ae(i)+3*2*Be(i))),
                           sum(i=0,h-1, 3*9^(h-1-i)*(Ao(i)+2*Bo(i) + 3*Ae(i)+3*2*Be(i)))  + Ae(h)+2*Be(h)
                          );
}
C(k) =
{
  my(h=floor(k/2));
     1/24*(9^k - 3^k) + if(k%2==0,
                           sum(i=0,h-1, 9^(h-1-i)*ABoe(i)),
                           sum(i=0,h-1, 3*9^(h-1-i)*ABoe(i))  + Ae(h)+2*Be(h)
                          );
}
C(k) =
{
  my(h=floor(k/2));
     1/24*(9^k - 3^k) + if(k%2==0,
                           sum(i=0,h-1, 9^(h-1-i)*ABoe(i)),
                           ABe(h) + 3* sum(i=0,h-1, 9^(h-1-i)*ABoe(i))
                          );
}
ABoe9(h) = sum(i=0,h-1, 9^(h-1-i)*ABoe(i));
ABoe9(h) = -3/7*(81^h-9^h)/(81-9) - 1/14*((-3)^h-9^h)/(-3-9);
ABoe9(h) = -1/168*(81^h-9^h) + 1/168*((-3)^h-9^h);
ABoe9(h) = -1/168*(81^h - (-3)^h);
C(k) =
{
  my(h=floor(k/2));
     1/24*(9^k - 3^k) + if(k%2==0,
                           ABoe9(h),
                           ABe(h) + 3* ABoe9(h)
                          );
}
C(k) =
{
  my(h=floor(k/2));
     1/24*9^k - 1/24*3^k + if(k%2==0,
                           -1/168*(9^k - (-3)^h),
                           -3/56 *(1/9*9^k - (-3)^h)
                          );
}
\\ 168 == 2^3 * 3 * 7
C(k) = 1/168*( 6*9^k - 7*3^k + 3^(2*(k%2))*(-3)^floor(k/2) );
C(k) = 1/168*( 6*9^k - 7*3^k + (-3)^(floor(k/2) + 2*(k%2)) );
C(k) = 1/168*( 6*9^k - 7*3^k + (-3)^(ceil(k/2) + (k%2)) );
C(k) = 1/168*( 6*9^k - 7*3^k + if(k%2==0, 1, -3)*(-3)^ceil(k/2) );
C(k) = 1/168*( 6*9^k - 7*3^k + if(k%2==0, 1, 9)*(-3)^floor(k/2) );

vector(12,k,C(k-1))

\\ 168 = 8*21        -12, 4, 16
A(k) = 1/168*( 10*9^k         + if(k%2==0,-10,  2)*(-3)^ceil(k/2) );
B(k) = 1/168*( -8*9^k         + if(k%2==0,  8,  4)*(-3)^ceil(k/2) );
C(k) = 1/168*(  6*9^k - 7*3^k + if(k%2==0,  1, -3)*(-3)^ceil(k/2) );

AplusB(k) = 1/168*(  2*9^k    + if(k%2==0, -2, -18)*(-3)^floor(k/2) );
AsubB(k)  = 6/168*(  3*9^k  -1/3* if(k%2==0, 9, 1)*(-3)^ceil(k/2) );
AsubB(k)  = 1/168*( 18*9^k    + if(k%2==0, -18, -2)*(-3)^ceil(k/2) );

for(k=0,20, test_equal(A(k)+B(k), AplusB(k)))
for(k=0,20, test_equal(A(k)-B(k), AsubB(k)))

\\ -10*i+8*j = -6*i-12*j
\\ -10*i+6*i = -12*j-8*j
\\ -4*i = -20*j
\\ i=5*j
A5plusB(k)  = 1/168*( 42*9^k    + if(k%2==0,-42,-42)*(-3)^floor(k/2) );
A5plusB(k)  = 1/4*( 9^k - (-3)^floor(k/2) );
for(k=0,20, test_equal(5*A(k)+B(k), A5plusB(k)))

f(k) = if(k%2==0,  1, -3);
A(k) = 1/168*( 10*9^k         + (-3*f(k)-7)*(-3)^ceil(k/2) );
B(k) = 1/168*( -8*9^k         + (f(k)+7)*(-3)^ceil(k/2) );
C(k) = 1/168*(  6*9^k - 7*3^k + f(k)*(-3)^ceil(k/2) );

A(k) = 1/168*( 10*9^k         + if(k%2==0,-10, -6)*(-3)^floor(k/2) );
B(k) = 1/168*( -8*9^k         + if(k%2==0,  8,-12)*(-3)^floor(k/2) );
C(k) = 1/168*(  6*9^k - 7*3^k + if(k%2==0,  1,  9)*(-3)^floor(k/2) );

A(k) = 1/168*( 10*9^k         + (-10 + 4*(k%2))*(-3)^floor(k/2) );
B(k) = 1/168*( -8*9^k         + (8 - 20*(k%2))*(-3)^floor(k/2) );
C(k) = 1/168*(  6*9^k - 7*3^k + (1 + 8*(k%2))*(-3)^floor(k/2) );

A(k) = 1/168*( 10*9^k         + (-8 -  2*(-1)^k)*(-3)^floor(k/2) );
B(k) = 1/168*( -8*9^k         + (-2 + 10*(-1)^k)*(-3)^floor(k/2) );
C(k) = 1/168*(  6*9^k - 7*3^k + ( 5 -  4*(-1)^k)*(-3)^floor(k/2) );

e=2
A(k) = 1/168*( 10*9^k         + (-8 + (-2 + e)*(-1)^k)*(-3)^floor(k/2) );
B(k) = 1/168*( -8*9^k         + (-2 + (10 - e)*(-1)^k)*(-3)^floor(k/2) );
C(k) = 1/168*(  6*9^k - 7*3^k + ( 5 + (-4-e/2)*(-1)^k)*(-3)^floor(k/2) );
E(k) = 1/168*(  0*9^k         + ( 0 +        e*(-1)^k)*(-3)^floor(k/2) );
\\ sin(alpha+deg( 60))^2, -sin(alpha)^2 + sin(alpha+deg(30))^2 + 1/2

A(k) = 1/168*( 10*9^k         + (-6 + ( 4)*(-1)^k)*(-3)^floor(k/2) );
B(k) = 1/168*( -8*9^k         + (-4 + ( 4)*(-1)^k)*(-3)^floor(k/2) );
C(k) = 1/168*(  6*9^k - 7*3^k + ( 4 + (-7)*(-1)^k)*(-3)^floor(k/2) );
E(k) = 1/168*(  0*9^k         + ( 2 +    6*(-1)^k)*(-3)^floor(k/2) );
\\  sin(alpha)^2 - sin(alpha+deg(30))^2 + sin(alpha+deg( 60))^2 - 1/2 = 0

\\ -10,2  
\\   8,4  
\\ 2/3*A + B+1/3*A
\\ -20/3,4/3  16/3,8/3   -18/3,-2/3
\\     -8       

{
for(i=0,11,
   talpha = deg(30*i);
   tlimit = 7;
   tmult = 8;
    print(nearly(vector(tlimit+1,k,my(k=k-1); inertia(k,talpha)*tmult)));
   \\ print(nearly(vector(tlimit+1,k,my(k=k-1); inertia_by_rec(k,talpha)*tmult)));
   print(nearly(vector(tlimit+1,k,my(k=k-1); inertia_by_ABC(k,talpha)*tmult)));
   )
}

