#!/usr/bin/perl -w

# Copyright 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Graph;
use Math::PlanePath::AlternateTerdragon;

BEGIN { unshift @INC, $FindBin::Bin }
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;
print "    % generated by tools/$FindBin::Script\n";
print "    \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::AlternateTerdragon->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $graph = Graph->new (undirected => 1);
my $segments = MyLineSegments->new;

my $n_mid = ($n_hi+1)/2;
my ($x_end,$y_end) = $path->n_to_xy($n_mid);
print "    \\coordinate (end) at ($x_end,$y_end);\n";

my @diagonal;
foreach my $n ($n_lo .. $n_mid-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($x2,$y2) = $path->n_to_xy($n+1);
  $segments->add($x,$y, $x2,$y2);
  $graph->add_edge("$x,$y", "$x2,$y2");
  if ($x == $y) { push @diagonal, [$x,$y]; }
}

my $diameters = MyLineSegments->new;
foreach my $p (@diagonal) {
  my ($x,$y) = @$p;
  while ($x != $x_end || $y != $y_end) {
    my ($x2,$y2) = ($y==0 ? ($x+2,$y) : ($x+1,$y-1));
    $graph->has_edge ("$x,$y", "$x2,$y2")
      or die "oops, no edge $x,$y to $x2,$y2";
    $segments->remove($x,$y, $x2,$y2);
    $diameters->add($x,$y, $x2,$y2);
    ($x,$y) = ($x2,$y2);
  }
}

$Printwrap = 4;
printwrap("\\draw[my grey] ");
$Printwrap += 2;
$segments->printwrap;
printwrap(";\n");

$Printwrap = 4;
printwrap("\\draw[my thick] ");
$Printwrap += 2;
$diameters->printwrap;
printwrap(";\n");

my $AltDiameterEndpoints_half = 0;
my @start_to_end = $graph->SP_Dijkstra("$x_end,$y_end", "0,0");
my $want_len = scalar(@start_to_end);
foreach my $v ($graph->vertices) {
  my @vertices = $graph->SP_Dijkstra("$x_end,$y_end", $v);
  if (@vertices == $want_len) {
    $AltDiameterEndpoints_half++;
  }
}
print "% GP-Test  AltDiameterEndpoints($level) == 2*$AltDiameterEndpoints_half\n";

print "    % end generated\n";
exit 0;
