#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018, 2019 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon 'xy_rotate_plus60','hull_and_end';
use MyLineSegments;
use MyPrintwrap ':all';


my $level = 6;
$level % 2 == 0 or die;

print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

sub transform {
  my ($x,$y) = @_;
  for (my $i = 0; $i < $level; $i+=2) {
    ($x,$y) = xy_rotate_plus60($x,$y);
  }
  return ($x,$y);
}
sub stringize {
  my ($x,$y) = @_;
  if ($y) {
    return "($x,{$y*sqrt(3)})";
  } else {
    return "($x,$y)";
  }
}


my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

my ($points,$ex,$ey) = hull_and_end($level);
$ex == $x_hi or die;
$ey == $y_hi or die;

# no curve segments where the hull sides drawing overlaps
my $removed = 0;
foreach my $i (0 .. $#$points) {
  eval {
    $segments->remove(@{$points->[$i-1]}, @{$points->[$i]});
    $removed++;
  };
}
$removed == 6 or die;  # triangular hull sides

$Printwrap = 2;
printwrap("\\draw[my grey] ");
$Printwrap = 4;
$segments->printwrap(transform => \&transform,
                     stringize => \&stringize);
printwrap(";\n");

my $num = 1;
my $dash = '';
foreach my $p (@$points) {
  my ($x,$y) = @$p;
  ($x,$y) = transform($x,$y);
  my $xy = stringize($x,$y);
  print "  \\node (P$num$dash) at $xy [my dot];\n";
  $num++;
  if ($num == 8) {
    $num = 1;
    $dash = 'dash';
  }
}

$num = 1;
$dash = '';
foreach my $p (@$points) {
  my ($x,$y) = @$p;
  print "% GP-Test-Last  P$num$dash($level) == $x/2 + $y*sqrt3i/2\n";
  $num++;
  if ($num == 8) {
    $num = 1;
    $dash = 'dash';
  }
}

{
  my ($x,$y) = transform(0,0);
  my $xy = stringize($x,$y);
  print "  \\coordinate (start) at $xy;\n";
}
{
  my ($x,$y) = transform($x_hi,$y_hi);
  my $xy = stringize($x,$y);
  print "  \\coordinate (end) at $xy;\n";
}

print "  % end generated\n";
exit 0;
