#!/usr/bin/perl -w

# Copyright 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use Lingua::EN::Numbers;

use FindBin;
use lib $FindBin::Bin;
use MyTerdragon 'chamfer','@dir6_to_dx','@dir6_to_dy', 'mul','xy_rotate_plus90';
use MyPrintwrap ':all';


my $max_level = 3;

print "  % generated by tools/$FindBin::Script\n";

my @segments = ([0,0, 0,0]);

foreach my $level (0 .. $max_level) {
  my $level_name = ucfirst Lingua::EN::Numbers::num2en($level);
  print "  \\begin{scope}[shift=(pos-$level),scale=\\MyScale$level_name]\n";
  my ($prev_x,$prev_y,$prev_dir);
  my @joins;
  $Printwrap = 4;
  foreach my $i (0 .. $#segments) {
    my $segment = $segments[$i];
    my ($x,$y, $dir,$rev) = @$segment;
    my $dx = $dir6_to_dx[$dir];
    my $dy = $dir6_to_dy[$dir];
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;
    if ($i != 0) {
      $x .= chamfer($dx, '\\MyChamfer');
      $y .= chamfer($dy, '\\MyChamfer');
    }
    if ($i != $#segments) {
      $x2 .= chamfer(-$dx, '\\MyArrowGap');
      $y2 .= chamfer(-$dy, '\\MyArrowGap');
    }
    printwrap(" \\draw[->] ($x,$y)--($x2,$y2);");
    if (defined $prev_x) {
      if ($rev) {
        push @joins, (" ($prev_x,$prev_y)--($x2,$y2)");
      } else {
        push @joins, (" ($prev_x,$prev_y)--($x,$y)");
      }
    }
    ($prev_x,$prev_y) = ($rev ? ($x,$y) : ($x2,$y2));
    $prev_dir = $dir;
  }
  printwrap_newline();

  if (@joins) {
    $Printwrap = 2;
    printwrap("\\draw");
    $Printwrap += 2;
    foreach my $join (@joins) {
      printwrap($join);
    }
    printwrap(";\n");
  }
  print "  \\coordinate (start-$level) at (0,0);\n";
  {
    my ($x,$y, $dir,$rev) = @{$segments[-1]};
    my $dx = $dir6_to_dx[$dir];
    my $dy = $dir6_to_dy[$dir];
    my $x2 = $x + $dx;
    my $y2 = $y + $dy;
    print "  \\coordinate (end-$level) at ($x2,$y2);\n";
  }

  print "  \\end{scope}\n";


  #          ^
  #           \
  #   ---> <---
  my @new_segments;
  foreach my $segment (@segments) {
    my ($x,$y, $dir, $rev) = @$segment;
    my $dx = $dir6_to_dx[$dir];
    my $dy = $dir6_to_dy[$dir];
    my ($lx,$ly) = xy_rotate_plus90($dx,$dy);

    ($x,$y) = mul($x,$y);
    my @these_new_segments
      = ([$x,$y, $dir, $rev],
         [$x+2*$dx, $y+2*$dy, ($dir+3)%6, 1-$rev],
         [$x+2*$dx, $y+2*$dy, ($dir+2)%6, $rev]);
    push @new_segments,
      ($rev ? reverse @these_new_segments : @these_new_segments);
  }
  @segments = @new_segments;
}
print "  % end generated\n";
exit 0;
