#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyPrintwrap ':all';
use MyLineSegments;

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 4;
print "  % generated by tools/$FindBin::Script\n";
print "  \\newcommand{\\MyLevel}{$level}\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);
my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
print "  % level=$level end at $x_hi,$y_hi\n";

sub n_is_hanging_triangle {
  my ($n) = @_;
  ### n_is_hanging_triangle(): $n
  $n+3 <= $n_hi or return 0;
  foreach my $i (0 .. 2) {
    $path->_UNDOCUMENTED__n_segment_is_right_boundary($n+$i)
      || $path->_UNDOCUMENTED__n_segment_is_right_boundary($n_hi-1 - ($n+$i))
        || return 0;
  }
  my ($x,$y) = $path->n_to_xy($n);
  my ($x3,$y3) = $path->n_to_xy($n+3);
  return ($x == $x3 && $y == $y3);
}

my $segments = MyLineSegments->new;
my $count_triangles = 0;
$Printwrap = 2;
printwrap("\\begin{scope}[my hanging]\n");
$Printwrap = 4;
for (my $n = $n_lo; $n < $n_hi; $n++) {
  if (n_is_hanging_triangle($n)) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($x1,$y1) = $path->n_to_xy($n+1);
    my ($x2,$y2) = $path->n_to_xy($n+2);
    printwrap(" \\draw ($x,$y)--($x1,$y1)--($x2,$y2)--cycle;");
    $n += 2;
    $count_triangles++;
  } else {
    $segments->add($path->n_to_xy($n),
                   $path->n_to_xy($n+1));
  }
}
printwrap_newline();
$Printwrap = 2;
printwrap("\\end{scope}\n");

printwrap("\\draw[my main] ");
$Printwrap = 4;
$segments->printwrap;
printwrap(";\n");

my $count_triangles_right = $count_triangles/2;

print "  \\newcommand{\\MyH}{$count_triangles}\n";
print "  \\newcommand{\\MyHR}{$count_triangles_right}\n";
print "% GP-Test-Last  H($level)==$count_triangles\n";
print "% GP-Test-Last  HR($level)==$count_triangles_right\n";
print "  % end generated\n";
exit 0;
