#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2018 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyPrintwrap ':all';


print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::TerdragonCurve->new;

my $level = 2;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

print "  \\begin{scope}[scale=1/6,yscale=sqrt(3)]\n";
{
  my ($x,$y) = $path->n_to_xy($n_hi);
  ($x,$y) = transform_to_horizontal($x,$y, $level);
  print "  \\node (start) at (0,0) [my big dot];\n";
  print "  \\node (end) at ($x,$y) [my big dot];\n";
}

$Printwrap = 2;
printwrap("\\draw[my line] (start)");
$Printwrap += 2;
foreach my $n ($n_lo+1 .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($sdx,$sdy) = $path->n_to_dxdy($n-1);
  my ($tdx,$tdy) = $path->n_to_dxdy($n);

  ($x,$y) = transform_to_horizontal($x,$y, $level);
  ($sdx,$sdy) = transform_to_horizontal($sdx,$sdy, $level);
  ($tdx,$tdy) = transform_to_horizontal($tdx,$tdy, $level);

  my $sx = $x . chamfer(-$sdx);
  my $sy = $y . chamfer(-$sdy);
  my $tx = $x . chamfer($tdx);
  my $ty = $y . chamfer($tdy);
  printwrap("--($sx,$sy)");
  printwrap("--($tx,$ty)");
}
printwrap("--(end);\n");
print "  \\end{scope}\n";

print "  % end generated\n";
exit 0;
