#!/usr/bin/perl -w

# Copyright 2014 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.004;
use strict;
use FindBin;
use Math::PlanePath::TerdragonCurve 117; # v.117 for level_to_n_range()

use FindBin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyTerdragon ':all';
use MyLineSegments;
use MyPrintwrap ':all';

# uncomment this to run the ### lines
# use Smart::Comments;


my $level = 3;
print "  % generated by tools/$FindBin::Script\n";

my $path = Math::PlanePath::TerdragonCurve->new;
my ($n_lo, $n_hi) = $path->level_to_n_range($level);

my $segments = MyLineSegments->new;
foreach my $n ($n_lo .. $n_hi-1) {
  $segments->add($path->n_to_xy($n),
                 $path->n_to_xy($n+1));
}

sub transform {
  my ($x,$y) = @_;
  my $k = $level;
  while ($k > 3) {
    $level -= 2;
    ($x,$y) = xy_rotate_minus60($x,$y);
  }
  $y = "{$y*sqrt(3)}";
  if ($k == 3) {
    ($x,$y) = ($y,-$x);  # rotate -90
    $k -= 3;
  }
  if ($k != 0) { die; }
  return ($x,$y);
}

$Printwrap = 2;
printwrap("\\draw[gray]");
$Printwrap = 3;
foreach my $line ($segments->lines) {
  my ($x1,$y1, $x2,$y2) = @$line;
  ($x1,$y1) = transform($x1,$y1);
  ($x2,$y2) = transform($x2,$y2);
  printwrap(" ($x1,$y1)--($x2,$y2)");
}
printwrap(";\n");

$Printwrap = 1;
foreach my $n ($n_lo .. $n_hi-1) {
  my ($x,$y) = $path->n_to_xy($n);
  my ($dx,$dy) = $path->n_to_dxdy($n);
  $x += $dx/2;
  $y += $dy/2;
  ($x,$y) = transform($x,$y);
  printwrap(" \\filldraw ($x,$y) circle;\n");
}

print "  % end generated\n";
exit 0;
